import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ProductGamesystem} from "../../../../game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../game-model/gamesystems/SimpleGamesystem";
import {
  MatCell, MatCellDef,
  MatColumnDef, MatHeaderCell, MatHeaderCellDef,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow,
  MatRowDef,
  MatTable,
  MatTableDataSource
} from "@angular/material/table";
import {SimpleState} from "../../../../game-model/gamesystems/SimpleState";
import {State} from "../../../../game-model/gamesystems/State";
import {NgForOf, NgIf} from "@angular/common";
import {ProductState} from "../../../../game-model/gamesystems/ProductState";
import {MatIcon} from "@angular/material/icon";
import {Gamesystem} from "../../../../game-model/gamesystems/Gamesystem";

@Component({
  selector: 'app-product-state-editor',
  standalone: true,
  imports: [
    MatTable,
    MatRow,
    MatRowDef,
    MatHeaderRow,
    MatHeaderRowDef,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderCellDef,
    MatCell,
    MatCellDef,
    NgForOf,
    NgIf,
    MatIcon
  ],
  templateUrl: './product-state-editor.component.html',
  styleUrl: './product-state-editor.component.scss'
})
export class ProductStateEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  @Output('onOpenGamesystemEditor') openGamesystemEditorEmitter = new EventEmitter<SimpleGamesystem>();
  displayedColumns: string[] = [];
  datasource = new MatTableDataSource();


  ngOnInit() {
    this.gamesystem!.generateFromChildsystems();
    this.generateColumnNamesRecursively(this.gamesystem!, "");
    this.displayedColumns.push('Initial');
    this.datasource.data = this.gamesystem!.states;
  }

  generateColumnNamesRecursively(gamesystem: ProductGamesystem, nestedColumnName: string) {
    gamesystem.innerGamesystems.forEach(innerGamesystem => {
      if(innerGamesystem instanceof SimpleGamesystem) {
        this.displayedColumns.push(nestedColumnName + innerGamesystem.componentName);
      } else {
        this.generateColumnNamesRecursively(innerGamesystem as ProductGamesystem, nestedColumnName + innerGamesystem.componentName + ".");
      }
    })
  }

  protected readonly SimpleState = SimpleState;

  getLeafState(state: State<any>, i: number) {
    return this.computeLeafStates(state)[i];
  }

  computeLeafStates(state: State<any>) {
    if(state instanceof SimpleState) {
      return [state];
    } else {
      const productState = state as ProductState;
      const leafStates: SimpleState[] = [];
      productState.innerStates.forEach(innerState => {
        this.computeLeafStates(innerState).forEach(leafState => leafStates.push(leafState));
      })
      return leafStates;
    }
  }

  clickOnInnerState(leafIndex: number) {
    const leaf_gamesystems = this.computeLeafGamesystems(this.gamesystem!);
    const clicked_gamesystem = leaf_gamesystems[leafIndex];
    this.openGamesystemEditorEmitter.emit(clicked_gamesystem);
  }

  computeLeafGamesystems(gamesystem: Gamesystem<any, any>) {
    if(gamesystem instanceof SimpleGamesystem) {
      return [gamesystem];
    } else {
      const product_gamesystem = gamesystem as ProductGamesystem;
      const leaf_gamesystems: SimpleGamesystem[] = [];
      product_gamesystem.innerGamesystems.forEach(innerGamesystem => {
        this.computeLeafGamesystems(innerGamesystem).forEach(leafGamesystem => leaf_gamesystems.push(leafGamesystem))
      })
      return leaf_gamesystems;
    }
  }
}
