import {Component, Input, OnInit} from '@angular/core';
import {GameModel} from "../../../../game-model/GameModel";
import {SimpleGamesystem} from "../../../../game-model/gamesystems/SimpleGamesystem";
import {
  MatCell,
  MatCellDef,
  MatColumnDef,
  MatHeaderCell,
  MatHeaderCellDef, MatHeaderRow, MatHeaderRowDef, MatRow, MatRowDef,
  MatTable,
  MatTableDataSource
} from "@angular/material/table";
import {SimpleTransition} from "../../../../game-model/gamesystems/SimpleTransition";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {SimpleState} from "../../../../game-model/gamesystems/SimpleState";

@Component({
  selector: 'app-simple-transition-editor',
  templateUrl: './simple-transition-editor.component.html',
  styleUrl: './simple-transition-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class SimpleTransitionEditorComponent implements OnInit {

  @Input() gamesystem: SimpleGamesystem | undefined
  displayedColumns: string[] = ["starting-state", "ending-state", "edit", "delete"];
  dataSource = new MatTableDataSource<SimpleTransition>();
  columnsToDisplayWithExpand = [...this.displayedColumns, 'expand'];
  expandedElement: SimpleTransition | null = null;
  editedTransition: SimpleTransition | undefined
  editedTransitionIndex = -1;

  defaultStartingState: SimpleState = new SimpleState("None", "");
  defaultEndingState: SimpleState = new SimpleState("None", "");

  transitionError: boolean = false;
  transitionStartingStateError = true;
  transitionEndingStateError = true;
  ngOnInit() {
    this.dataSource.data = this.gamesystem!.transitions;
  }

  addTransition() {
    this.editedTransition = new SimpleTransition(this.defaultStartingState, this.defaultEndingState);
    const updatedData = this.dataSource.data;
    updatedData.push(this.editedTransition);
    this.dataSource.data = updatedData;
    this.editedTransitionIndex = this.dataSource.data.length;
  }

  validateEditedTransition() {
    this.transitionError = false;
    this.transitionStartingStateError = false;
    this.transitionEndingStateError = false;
    if(this.editedTransition!.startingState === this.editedTransition!.endingState) {
      this.transitionError = true;
    } else if(this.editedTransition!.startingState == this.defaultStartingState) {
      this.transitionStartingStateError = true;
    } else if(this.editedTransition!.endingState == this.defaultEndingState) {
      this.transitionEndingStateError = true;
    }
  }

  finishEditing() {
    this.validateEditedTransition();
    if(this.transitionError || this.transitionStartingStateError || this.transitionEndingStateError) {
      return;
    }
    this.gamesystem?.createTransition(this.editedTransition!.startingState, this.editedTransition!.endingState);
    this.dataSource.data = this.gamesystem!.transitions;
    this.editedTransition = undefined;
  }

  deleteTransition(transition: SimpleTransition) {
    if(this.gamesystem!.parentGamesystem == undefined) {
      this.gamesystem!.removeTransition(transition);
      this.dataSource.data = this.gamesystem!.transitions;
    }
  }
}
